#include "stdafx.h"
#include "DrawingArea.h"

CDrawingArea::CDrawingArea(void)
:	x_min(0.0f)
,	x_max(0.0f)
,	y_min(0.0f)
,	y_max(0.0f)
,	x_step(0.0f)
,	y_step(0.0f)
,	strXUnit(_T("m"))
,	strYUnit(_T("m"))
,	m_initialized(false)
,	m_varraySize(0)
{
}

CDrawingArea::~CDrawingArea(void)
{
	DeleteObject(m_blackPen) ;
	DeleteObject(m_grayPen) ;
	DeleteObject(m_font);
}

BEGIN_MESSAGE_MAP(CDrawingArea, CStatic)
	ON_WM_PAINT()
END_MESSAGE_MAP()


void CDrawingArea::SetBounds(double xmin, double xmax, double ymin, double ymax){

	x_min = xmin ; x_max = xmax ;
	y_min = ymin ; y_max = ymax ;

	GetWindowRect( m_rect ) ;
		m_height = m_rect.Height() ;
		m_width = m_rect.Width() ;

	// we compute the scale :
	double cx = x_max-x_min ;
	x_step = cx/8 ;
	double cy = y_max-y_min ;
	y_step = cy/5 ;

	if( !m_initialized ){

		m_blackPen = CreatePen( PS_SOLID, 2, RGB(0,0,0) );
		m_grayPen = CreatePen( PS_SOLID, 0, RGB(200,200,200) );
		m_redPen = CreatePen( PS_SOLID, 1, RGB(255,0,0) );

		m_font = CreateFontA(10,6,0,0,FW_THIN,FALSE,FALSE,FALSE,DEFAULT_CHARSET,OUT_DEFAULT_PRECIS,CLIP_DEFAULT_PRECIS,ANTIALIASED_QUALITY,FF_DONTCARE, "Arial") ;
		m_initialized = true ;
	}
}

int CDrawingArea::GetLocalX( double x_value ){

	int calc ;

	calc = ((int)(m_width*((x_value-x_min)/(x_max-x_min)))) ;

	return calc ;
}

int CDrawingArea::GetLocalY( double y_value ){

	int calc ;

	calc = m_height - ((int)(m_height*((y_value-y_min)/(y_max-y_min)))) ;

	return calc ;
}


void CDrawingArea::OnPaint()
{

	CPaintDC pTempDC(this);
	HDC m_hdc = pTempDC.m_hDC ;
	double i ;
	int _x, _y ;

	// draw the background :
	//**********************
	SelectObject(m_hdc,GetStockObject(DC_PEN));
	SetDCPenColor(m_hdc,RGB(00,00,0xff));
	Rectangle(m_hdc,0,0,m_width,m_height) ;

	// draw the grid :
	//****************
	
	SelectObject(m_hdc,m_grayPen);
		for(i=x_min; i<x_max; i+=x_step){
			MoveToEx(m_hdc, GetLocalX(i), 0, NULL) ;
			LineTo(m_hdc, GetLocalX(i), m_height) ;
		}
		for(i=y_min; i<y_max; i+=y_step){
			MoveToEx(m_hdc, 0, GetLocalY(i), NULL) ;
			LineTo(m_hdc, m_width, GetLocalY(i)) ;
		}

	// and some labels :
	//******************
	SelectObject(m_hdc,m_font);
		tmp_rect.SetRect( 0, 2, 0, 17);
		int _i ;
		
		for(i=x_min+x_step; i<x_max; i+=x_step){
			_i = GetLocalX(i) ;
			tmp_rect.left = _i-20 ;
			tmp_rect.right = _i+20 ;
			tmp_str.Format("%3.2f%s",i,strXUnit);
			DrawTextA(m_hdc,LPCSTR(tmp_str),7, tmp_rect,DT_INTERNAL|DT_LEFT);
		}
		tmp_rect.SetRect( 2, 0, 42, 15);
		for(i=y_min+y_step; i<y_max; i+=y_step){
			_i = GetLocalY(i) ;
			tmp_rect.top = _i ;
			tmp_rect.bottom = _i+15 ;
			tmp_str.Format("%3.2f%s",i,strYUnit);
			DrawTextA(m_hdc,LPCSTR(tmp_str),7, tmp_rect,DT_INTERNAL|DT_LEFT);
		}

	// draw X & Y axis :
	//******************
	SelectObject(m_hdc,m_blackPen);

		m_iTmp = GetLocalY(0.0f) ;
		MoveToEx(m_hdc, GetLocalX(x_min), m_iTmp, NULL) ;
		LineTo(m_hdc, GetLocalX(x_max), m_iTmp) ;

		m_iTmp = GetLocalX(0.0f) ;
		MoveToEx(m_hdc, m_iTmp, GetLocalY(y_min), NULL) ;
		LineTo(m_hdc, m_iTmp, GetLocalY(y_max)) ;

	// And the data :
	//****************
	if( m_varraySize>0 ){

		m_it=m_varray.begin() ;

			_x = GetLocalX( (*m_it).first ) ,
			_y = GetLocalY( (*m_it).second ) ;

		SelectObject(m_hdc, m_redPen);
		Rectangle(m_hdc,_x-1,_y-1,_x+2,_y+2) ;
		MoveToEx(m_hdc, _x, _y, NULL) ;
		
		for(m_it++; m_it!=m_varray.end(); m_it++){
				
				_x = GetLocalX( (*m_it).first ) ;
				_y = GetLocalY( (*m_it).second ) ;
				LineTo(m_hdc, _x, _y) ;
				Rectangle(m_hdc,_x-1,_y-1,_x+2,_y+2) ;
			}
	}


	ReleaseDC(&pTempDC);
}

void CDrawingArea::DrawLine( double x1, double y1, double x2, double y2){

	CPaintDC pTempDC(this);
	HDC m_hdc = pTempDC.m_hDC ;

	// draw X & Y axis :
	//******************
	SelectObject(m_hdc,m_redPen);

		m_iTmp = GetLocalY(0.0f) ;
		MoveToEx(m_hdc, GetLocalX(x1), GetLocalY(y1), NULL) ;
		LineTo(m_hdc, GetLocalX(x2), GetLocalY(y2)) ;

	ReleaseDC(&pTempDC);
}

void CDrawingArea::SetUnits(CString Xunit, CString Yunit){

	strXUnit = Xunit ;
	strYUnit = Yunit ;
}

void CDrawingArea::SetData( t_mapXY varray ){

	m_varray.clear() ;

	t_mapXY::iterator it ;
	Double_Pair tmp ;

	double xmin=100000.0f,xmax=-100000.0f,ymin=100000.0f,ymax=-100000.0f ;

	for(it = varray.begin(); it!=varray.end(); it++){
		
		tmp = *it ;
		m_varray.insert( *it ) ;
		if(tmp.first<xmin) xmin = tmp.first ;
		if(tmp.first>xmax) xmax = tmp.first ;
		if(tmp.second<ymin) ymin = tmp.second ;
		if(tmp.second>ymax) ymax = tmp.second ;
	}

	m_varraySize = m_varray.size() ;

	double
		marge1 = (xmax-xmin) ,
		marge2 = (ymax-ymin) ;
	SetBounds( xmin-marge1, xmax+marge1, ymin-marge2, ymax+marge2) ;
}